#!/usr/bin/python -u

# DIYBlog - a simple customisable blogging system for FTP-only web space.
# Copyright(C) 2005 Andy Balaam
# Contact: andybalaam@artificialworlds.net
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

import feedparser, sys, time, datetime
import xml.dom.minidom

# For older versions of python
True = 1
False = 0

if len( sys.argv ) != 2:
    print(
        """Usage:
            
            diyblog.py config_file.py
            """ )
    sys.exit(1)

# Set some empty defaults
input_feeds      = ()
rss_template     = ""
root_dir         = ""
html_template    = ""
html_output      = ""
rss_output       = ""
rss_output_title = ""
rss_output_link  = ""
rss_output_desc  = ""
rss_output_lang  = ""

# Get the config values
execfile( sys.argv[1] )

# --------------------------------------------------------------------------

chars_to_replace = ( " ", "'", ":", "*", "@", ",", ".", "!",
                     "$", "%", "^", "&", "(", ")", "+", "=", "{",
                     "}", "[", "]", ";", "<", ">", "/", "?", "|",
                     "\\" );

# --------------------------------------------------------------------------

def sanitise( st ):
    for char in chars_to_replace:
        st = st.replace( char, "_" )
    return st;

# --------------------------------------------------------------------------

def generalSubs( st, feed ):
    
    return st.replace( "$title", feed.title ).replace( "$rss_feed", rss_output )

# --------------------------------------------------------------------------

def itemSubs( st, entry ):
    
    title = ""
    link = ""
    site_title = ""
    site_link = ""
    date = ""
    desc = ""
    ajb_id = ""
    
    if "title" in entry:
        title = entry.title
    if "link" in entry:
        link = entry.link
    if "site_title" in entry:
        site_title = entry.site_title
    if "site_link" in entry:
        site_link = entry.site_link
    if "date_parsed" in entry:
        date = time.strftime( "%A %d %B %Y %H:%M %Z", entry.date_parsed )
    if "description" in entry:
        desc = entry.description
    if "ajb_id" in entry:
        ajb_id = entry.ajb_id
    
    return st.replace( "$item_title", title ).replace( "$item_link", link ).replace( "$item_site_title", site_title ).replace( "$item_site_link", site_link ).replace( "$item_date", date ).replace( "$item_description", desc ).replace( "$item_id", ajb_id )

# --------------------------------------------------------------------------

def appendSimpleChild( doc, parent, name, text ):
    element = doc.createElement( name )
    element.appendChild( doc.createTextNode( text ) )
    parent.appendChild( element )

# --------------------------------------------------------------------------

def writeToXML( filename, parsed_rss ):
    
    feed = parsed_rss.feed
    doc = xml.dom.minidom.Document()
    
    rss = doc.createElement( "rss" )
    rss.setAttribute( "version", "2.0" )
    doc.appendChild( rss )
    channel = doc.createElement( "channel" )
    rss.appendChild( channel )
    appendSimpleChild( doc, channel, "title", feed.title )
    appendSimpleChild( doc, channel, "link", feed.link )
    appendSimpleChild( doc, channel, "description", feed.description )
    appendSimpleChild( doc, channel, "language", feed.language )
    if "modified" in feed:
        appendSimpleChild( doc, channel, "pubDate", feed.modified )
    if "generator" in feed:
        gen = feed.generator
    else:
        gen = "DIYBlog http://www.artificialworlds.net/blog/diyblog/diyblog.html"
    appendSimpleChild( doc, channel, "generator", gen )
    
    if "entries" in parsed_rss:
        for entry in parsed_rss.entries:
            item = doc.createElement( "item" )
            channel.appendChild( item )
            appendSimpleChild( doc, item, "title", entry.title )
            appendSimpleChild( doc, item, "pubDate", entry.date )
            appendSimpleChild( doc, item, "description", entry.description )
            appendSimpleChild( doc, item, "link", entry.link )
            if "guid" in entry:
                appendSimpleChild( doc, item, "guid", entry.guid )
    else:
        print "No entries!"
        sys.exit(2)
    
    output_rss_file = file( filename, "w" )
    output_rss_file.write( doc.toxml() )
    output_rss_file.close()
    
# --------------------------------------------------------------------------

def entry_cmp(x, y):
    if y.date_parsed > x.date_parsed:
        return 1
    else:
        return -1
        
# --------------------------------------------------------------------------

def complete_rss( input_parsed, blog_home ):
    touched_rss = False
    for entry in input_parsed.entries:
        if entry.date_parsed > input_parsed.feed.modified_parsed:
            input_parsed.feed.modified_parsed = entry.date_parsed
        
        nm = "%s_%s" % ( sanitise( entry.title ), time.strftime( "%Y%m%d%H%M%S", entry.date_parsed ) )
        entry.ajb_id = nm
        
        if not( entry.has_key("link") and entry.has_key("guid") ):
            
            if not entry.has_key("link"):
                entry.link = "%s#%s" % ( blog_home, nm )
            
            if not entry.has_key("guid"):
                entry.guid = "%s#%s" % ( blog_home, nm )
            
            touched_rss = True
            
    return touched_rss

# --------------------------------------------------------------------------

def readTemplate( file_name ):
    template_file = file( file_name )

    top_template = ""
    item_template = ""
    bottom_template = ""
    
    state = "top"
    for line in template_file:
        if line.find( "<!-- BEGIN ITEM TEMPLATE -->" ) > -1:
            state = "middle"
        elif line.find( "<!-- END ITEM TEMPLATE -->" ) > -1:
            state = "bottom"
        else:
            if state == "top":
                top_template += line
            elif state == "middle":
                item_template += line
            else:
                bottom_template += line
    
    return (top_template, item_template, bottom_template )

def writeToHTML( html_output, rss, html_template ):
    ( top_template, item_template, bottom_template ) = readTemplate( html_template )
    
    output_file = file( html_output, "w" )

    output_file.write( generalSubs( top_template, rss.feed ) )
    
    for entry in rss.entries:
        output_file.write( itemSubs( item_template, entry ) )
    
    output_file.write( generalSubs( bottom_template, rss.feed ) )
    
    output_file.close()

# --------------------------------------------------------------------------

# Make an output rss:
if rss_output:
    output_rss = feedparser.parse( "" )
    output_rss.feed.modified_parsed = ( 1977, 01, 31, 04, 32, 00 )
    output_rss.feed.title = rss_output_title
    output_rss.feed.link = rss_output_link
    output_rss.feed.description = rss_output_desc
    output_rss.feed.language = rss_output_lang
else:
    output_rss = ""

for fd in input_feeds:
    # Read in this input file
    if "editable" in fd and fd["editable"]:
        url = root_dir + fd["rss"]
    else:
        url = fd["rss"]
    input_parsed = feedparser.parse( url )
    
    # If this is an editable file, edit it
    if "editable" in fd and fd["editable"]:
        touched_rss = complete_rss( input_parsed, fd["home"] )
        if touched_rss:
            writeToXML( fd["rss"], input_parsed )
    
    # Copy this feed into the output
    if rss_output:
        for entry in input_parsed.entries:
            entry.site_title = fd["name"]
            entry.site_link = fd["home"]
            if "modified" not in output_rss.feed:
                output_rss.feed.modified = entry.date
            elif entry.date_parsed > output_rss.feed.modified_parsed:
                output_rss.feed.modified_parsed = entry.date_parsed
                
            output_rss.entries.append( entry )
    else:
        writeToHTML( fd["html_output"], input_parsed, fd["html_template"] )

# Sort the entries in date order and write out
if rss_output:
    output_rss.entries.sort( entry_cmp )
    writeToXML( rss_output, output_rss )
    writeToHTML( html_output, output_rss, html_template )
    





